"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("rxjs/add/observable/fromPromise");
require("rxjs/add/operator/map");
var core_1 = require("@angular/core");
var http_1 = require("@angular/http");
var app_version_1 = require("@ionic-native/app-version");
var in_app_browser_1 = require("@ionic-native/in-app-browser");
var storage_1 = require("@ionic/storage");
var ionic_angular_1 = require("ionic-angular");
var rxjs_1 = require("rxjs");
var semver = require("semver");
var i18n_1 = require("./i18n");
var manup_config_1 = require("./manup.config");
/**
 * DI InjectionToken for optional ngx-translate
 */
exports.TRANSLATE_SERVICE = new core_1.InjectionToken('manup:translate');
var STORAGE_KEY = 'com.nextfaze.ionic-manup';
/**
 * The types of alerts we may present
 */
var AlertType;
(function (AlertType) {
    /**
       * A mandatory update is required
       */
    AlertType[AlertType["MANDATORY"] = 0] = "MANDATORY";
    /**
       * An optional update is available
       */
    AlertType[AlertType["OPTIONAL"] = 1] = "OPTIONAL";
    /**
       * The app is disabled
       */
    AlertType[AlertType["MAINTENANCE"] = 2] = "MAINTENANCE";
    /**
       * Nothing to see here
       */
    AlertType[AlertType["NOP"] = 3] = "NOP";
})(AlertType = exports.AlertType || (exports.AlertType = {}));
var ManUpService = (function () {
    function ManUpService(config, http, alert, platform, iab, AppVersion, translate, storage) {
        this.config = config;
        this.http = http;
        this.alert = alert;
        this.platform = platform;
        this.iab = iab;
        this.AppVersion = AppVersion;
        this.translate = translate;
        this.storage = storage;
        /**
           * True if there is an alert already displayed. Used to prevent multiple alerts
           * being presented on top of one another
           */
        this.inProgress = false;
        // load the translations unless we've been told not to
        if (this.translate && !this.config.externalTranslations) {
            for (var _i = 0, i18n_2 = i18n_1.i18n; _i < i18n_2.length; _i++) {
                var lang = i18n_2[_i];
                this.translate.setTranslation(lang.lang, lang.translations, true);
            }
        }
    }
    /**
       * Begins the manup check process.
       *
       * @Returns a promise that resolves if the app is able to continue.
       */
    ManUpService.prototype.validate = function () {
        var _this = this;
        if (!this.inProgress) {
            this.inProgress = true;
            this.currentPromise = new Promise(function (resolve, reject) {
                _this.platform.ready().then(function () {
                    _this.metadata()
                        .map(function (response) { return _this.getPlatformData(response); })
                        .subscribe(function (metadata) {
                        _this.evaluate(metadata).then(function (alert) {
                            switch (alert) {
                                case AlertType.NOP:
                                    resolve();
                                    break;
                                default:
                                    return _this.presentAlert(alert, metadata);
                            }
                        }, function (error) { return resolve(); });
                    });
                });
            });
        }
        return this.currentPromise;
    };
    /**
     * Evaluates what kind of update is required, if any.
     *
     * Returns a promise that resolves with an alert type.
     */
    ManUpService.prototype.evaluate = function (metadata) {
        if (!metadata.enabled) {
            return Promise.resolve(AlertType.MAINTENANCE);
        }
        return this.AppVersion.getVersionNumber().then(function (version) {
            if (semver.lt(version, metadata.minimum)) {
                return AlertType.MANDATORY;
            }
            else if (semver.lt(version, metadata.latest)) {
                return AlertType.OPTIONAL;
            }
            return AlertType.NOP;
        });
    };
    /**
     * Fetches the remote metadata and returns an observable with the json
     */
    ManUpService.prototype.metadata = function () {
        var _this = this;
        return this.http
            .get(this.config.url)
            .map(function (response) { return response.json(); })
            .map(function (response) {
            if (_this.storage) {
                _this.saveMetadata(response).catch(function () { });
            }
            return response;
        })
            .catch(function (err) {
            if (_this.storage) {
                return _this.metadataFromStorage();
            }
            return err;
        });
    };
    /**
       * Gets the version metadata from storage, if available.
       *
       * @private
       * @throws An error if the service was instantiated without a Storage component.
       * @returns {Promise<any>} That resolves with the metadata
       *
       * @memberOf ManUpService
       */
    ManUpService.prototype.metadataFromStorage = function () {
        if (this.storage) {
            return rxjs_1.Observable.fromPromise(this.storage.get(STORAGE_KEY + '.manup')).map(function (v) { return JSON.parse(v); });
        }
        else {
            throw new Error('Storage not configured');
        }
    };
    /**
       *
       * Saves the metadata to storage.
       *
       * @private
       * @param {ManUpData} metadata The metadata to store
       * @throws {Error} if storage if not configured
       * @returns {Promise<any>} A promise that resolves when the save succeeds
       *
       * @memberOf ManUpService
       */
    ManUpService.prototype.saveMetadata = function (metadata) {
        if (this.storage) {
            return this.storage.set(STORAGE_KEY + '.manup', JSON.stringify(metadata));
        }
        else {
            throw new Error('Storage not configured');
        }
    };
    /**
       * Returns the branch of the metadata relevant to this platform
       */
    ManUpService.prototype.getPlatformData = function (metadata) {
        if (this.platform.is('ios')) {
            return metadata.ios;
        }
        if (this.platform.is('android')) {
            return metadata.android;
        }
        if (this.platform.is('windows')) {
            return metadata.windows;
        }
        throw new Error('Unknown platform');
    };
    /**
       * Presents an update alert.
       *
       * @param type The type of alert to show
       * @param platformData The metadata for the platform
       *
       * @returns A promise that resolves when this whole thing is over.
       */
    ManUpService.prototype.presentAlert = function (type, platformData) {
        switch (type) {
            case AlertType.MANDATORY:
                return this.presentMandatoryUpdate(platformData);
            case AlertType.OPTIONAL:
                return this.presentOptionalUpdate(platformData);
            case AlertType.MAINTENANCE:
                return this.presentMaintenanceMode();
        }
    };
    /**
       * Displays a maintenance mode alert.
       *
       * @returns a promise that will never resolve, because the app should not continue
       */
    ManUpService.prototype.presentMaintenanceMode = function () {
        var _this = this;
        return this.AppVersion.getAppName().then(function (name) {
            return new Promise(function (resolve, reject) {
                var alert = _this.alert.create({
                    enableBackdropDismiss: false,
                    title: _this.translate
                        ? _this.translate.instant('manup.maintenance.title', { app: name })
                        : name + " Unavailable",
                    subTitle: _this.translate
                        ? _this.translate.instant('manup.maintenance.text', { app: name })
                        : name + " is currently unavailable. Please check back later"
                });
                alert.present();
            });
        });
    };
    /**
       * Displays a mandatory update alert.
       *
       * @returns a promise that will never resolve, because the app should not continue
       */
    ManUpService.prototype.presentMandatoryUpdate = function (platformData) {
        var _this = this;
        return this.AppVersion.getAppName().then(function (name) {
            return new Promise(function (resolve, reject) {
                var alert = _this.alert.create({
                    enableBackdropDismiss: false,
                    title: _this.translate
                        ? _this.translate.instant('manup.mandatory.title', { app: name })
                        : 'Update Required',
                    subTitle: _this.translate
                        ? _this.translate.instant('manup.mandatory.text', { app: name })
                        : "An update to " + name + " is required to continue.",
                    buttons: [
                        {
                            text: _this.translate ? _this.translate.instant('manup.buttons.update') : 'Update',
                            handler: function () {
                                _this.iab.create(platformData.url, '_system');
                                return false;
                            }
                        }
                    ]
                });
                alert.present();
            });
        });
    };
    /**
       * Displays an optional update alert.
       *
       * @returns a promise that will resolves if the user selects 'not now'
       */
    ManUpService.prototype.presentOptionalUpdate = function (platformData) {
        var _this = this;
        return this.AppVersion.getAppName().then(function (name) {
            return new Promise(function (resolve, reject) {
                var alert = _this.alert.create({
                    enableBackdropDismiss: false,
                    title: _this.translate
                        ? _this.translate.instant('manup.optional.title', { app: name })
                        : 'Update Available',
                    subTitle: _this.translate
                        ? _this.translate.instant('manup.optional.text', { app: name })
                        : "An update to " + name + " is available. Would you like to update?",
                    buttons: [
                        {
                            text: _this.translate ? _this.translate.instant('manup.buttons.later') : 'Not Now',
                            handler: function () {
                                resolve();
                            }
                        },
                        {
                            text: _this.translate ? _this.translate.instant('manup.buttons.update') : 'Update',
                            handler: function () {
                                _this.iab.create(platformData.url, '_system');
                                return false;
                            }
                        }
                    ]
                });
                alert.present();
            });
        });
    };
    return ManUpService;
}());
ManUpService.decorators = [
    { type: core_1.Injectable },
];
/** @nocollapse */
ManUpService.ctorParameters = function () { return [
    { type: manup_config_1.ManUpConfig, },
    { type: http_1.Http, },
    { type: ionic_angular_1.AlertController, },
    { type: ionic_angular_1.Platform, },
    { type: in_app_browser_1.InAppBrowser, },
    { type: app_version_1.AppVersion, },
    { type: undefined, decorators: [{ type: core_1.Optional }, { type: core_1.Inject, args: [exports.TRANSLATE_SERVICE,] },] },
    { type: storage_1.Storage, decorators: [{ type: core_1.Optional },] },
]; };
exports.ManUpService = ManUpService;
//# sourceMappingURL=manup.service.js.map